#!/usr/bin/perl
use strict;
use Device::SerialPort;
use IO::Socket;
use IO::Select;

print "Starting up server\n";

my ($serialPort);
my ($read_set,$serverSocket);

$read_set = new IO::Select();
setUpSocket();
setupSerialPort();


print "Listening\n";
while (1) {
	# finds and socket of FileHandle with waiting data in the buffer;
	my @rh_set;
	my ($socket, $handle);

	@rh_set = $read_set->can_read(); # whole seconds timeout
	# wait until somebody sqwaks;

	next unless (defined $rh_set[0] );	# fixes issue when the BS2 cycles power
	
	
	#print "$rh_set[0] file handle(s) has data\n";
	
	for $handle (@rh_set) {
		# if the serverSocket is ready, accept a new connection
		if($handle == $serverSocket) {
			my $new = $serverSocket->accept();
			#print "created new socket $new\n";

			$read_set->add($new);
			print $new->fileno . ": connected\n";
			
		} else {
			my $buffer = "";
			# read from the other sockets
			
			
			#serial port has something to say
			if ($handle == \*SERIAL){	
				#print "read serial\n";
				my ($count,$saw) = $serialPort->read(2);		# get payload in bytes
				$saw +=0; #converts a string to a number;
				#print "saw: $saw\n";
				($count,$buffer) = $serialPort->read($saw); 	# will read _up to_ 255 chars
				#print "\"$buffer\", length: ".length($buffer) ." bytes\n";
				
				writeToFlash($buffer);
				
				
			}else{
				#must be the flash sockets with data on them
				$handle->recv($buffer,80);
				chop $buffer; # removes the \0 from the flash string
				
				#print "in:\"$buffer\", length: ".length($buffer) ." bytes\n";
				my $limit = 0;

				if(!$buffer) {
					# close the connection if recv() fails.
					print $handle->fileno . ": disconnected\n";
					$read_set->remove($handle);
					$handle->close;
					
				}else{				
					my @data = split /:/,$buffer;
				
					handshake:
			
					#print "\nwake up serial\n";
					$serialPort->write("X");
					#print "wait for reply\n";
					my ($count,$saw) = $serialPort->read(1);# get response character
					#print "in:\"$saw\", length: ".length($saw) ." bytes\n";
					
					if ($saw eq "X"){
						#print "get an X back, BS2 is waiting to read now...\n";
						
						$serialPort->write("$data[0] \n$data[1]\n");
						
						#print "wait for a Y from BS2 \n";

						my ($count,$saw) = $serialPort->read(1);# get payload in bytes
						
						if ($saw eq "Y"){
							#print "got an 'OK' from the BS2 \n";
							
						}else {
							#print "got nothing back, try again $limit\n";
							$limit++;
							sleep .1;
							goto handshake unless ($limit >5);
						}
						
					}else{
						#print "got nothing back, try again $limit\n";
						$limit++;
						sleep .1;
						goto handshake unless ($limit >5);
					}
					
				}
			}
		}		
	}
}
sub writeToFlash{
	my $outgoingSocket;
	my $data = $_[0];
	for $outgoingSocket ($read_set->handles) {
		next if($outgoingSocket == $serverSocket);	# don't send to yourself
		next if($outgoingSocket == \*SERIAL);			# 
		
		$outgoingSocket->send($data."\0") or do {
			print $outgoingSocket->fileno . ": disconnected\n";		
			$read_set->remove($outgoingSocket);
			$outgoingSocket->close;
		}
	}
}


sub setupSerialPort{
	#capture the serial port
	my $portName = "/dev/tty.USA19H1b12P1.1";
	$serialPort = tie (*SERIAL, 'Device::SerialPort', $portName) || die "error\n";
	$read_set->add(\*SERIAL);
	
	# setup communications;
	$serialPort->baudrate(9600);
	$serialPort->databits(8);
	$serialPort->parity("none");
	$serialPort->stopbits(1);	
	$serialPort->read_char_time(0);     # don't wait for each character
	$serialPort->read_const_time(50); 	# milliseconds per unfulfilled "read" call
	#print "setup serialPort $serialPort\n";
}

sub setUpSocket{
	$serverSocket = IO::Socket::INET->new(
						PeerAddr => '192.168.10.127',
						PeerPort => 7070,
						LocalAddr => '192.168.10.127',
						LocalPort => 7070,
						Proto => 'tcp',
						Listen => 1,
						Reuse => 1) or die $!;
						

	$read_set->add($serverSocket);
}
